==> spring-boot-ext-1.3.39-sources.jar: cn/com/duibaboot/ext/autoconfigure/perftest/WebSocketPerfTestConfiguration.java <==
package cn.com.duibaboot.ext.autoconfigure.perftest;

import cn.com.duiba.boot.perftest.InternalPerfTestContext;
import cn.com.duiba.boot.perftest.PerfTestUtils;
import cn.com.duibaboot.ext.autoconfigure.core.SpecifiedBeanPostProcessor;
import com.alibaba.ttl.TransmittableThreadLocal;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeansException;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnResource;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;
import org.springframework.messaging.simp.SimpMessageHeaderAccessor;
import org.springframework.messaging.simp.config.ChannelRegistration;
import org.springframework.messaging.support.ExecutorChannelInterceptor;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.config.annotation.AbstractWebSocketMessageBrokerConfigurer;
import org.springframework.web.socket.config.annotation.StompEndpointRegistry;
import org.springframework.web.socket.server.HandshakeInterceptor;
import org.springframework.web.socket.server.support.WebSocketHandlerMapping;
import org.springframework.web.socket.server.support.WebSocketHttpRequestHandler;
import org.springframework.web.socket.sockjs.support.SockJsHttpRequestHandler;
import org.springframework.web.socket.sockjs.transport.TransportHandlingSockJsService;

import java.util.Map;

/**
 * websocket线上压力测试自动配置
 *
 * Created by wenqi.huang on 2016/11/24.
 */
@Configuration
@ConditionalOnResource(resources = "/autoconfig/perftest.properties")
@ConditionalOnClass({TransmittableThreadLocal.class,
        WebSocketHandlerMapping.class,
        AbstractWebSocketMessageBrokerConfigurer.class})
@Slf4j
public class WebSocketPerfTestConfiguration {

    /**
     * websocket 请求加入压测标记(只支持spring-websocket的socketjs messagebroker开发方式)
     * @return
     */
    @Bean
    public static SpecifiedBeanPostProcessor<WebSocketHandlerMapping> webSocketHandlerMappingSpecifiedBeanPostProcessor(){//NOSONAR
        return new SpecifiedBeanPostProcessor<WebSocketHandlerMapping>() {
            @Override
            public Class<WebSocketHandlerMapping> getBeanType() {
                return WebSocketHandlerMapping.class;
            }

            @Override
            public Object postProcessBeforeInitialization(WebSocketHandlerMapping bean, String beanName) throws BeansException {
                return bean;
            }

            @Override
            public Object postProcessAfterInitialization(WebSocketHandlerMapping bean, String beanName) throws BeansException {
                for(Object rh : bean.getUrlMap().values()){
                    if (rh instanceof SockJsHttpRequestHandler) {
                        SockJsHttpRequestHandler rh1 = (SockJsHttpRequestHandler)rh;
                        ((TransportHandlingSockJsService) rh1.getSockJsService())
                                .getHandshakeInterceptors().add(0, new PerfTestHandshakeInterceptor());
                    } else if(rh instanceof WebSocketHttpRequestHandler) {
                        WebSocketHttpRequestHandler rh1 = (WebSocketHttpRequestHandler)rh;
                        rh1.getHandshakeInterceptors().add(0, new PerfTestHandshakeInterceptor());
                    } else {
                        log.warn("暂时不支持对此websocket处理类：【{}】的压测, 请使用sockjs或stomp", rh.getClass().toString());
                    }
                }
                return bean;
            }

            @Override
            public int getOrder() {
                return 0;
            }
        };
    }

    public static class PerfTestHandshakeInterceptor implements HandshakeInterceptor {
        @Override
        public boolean beforeHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Map<String, Object> attributes) throws Exception {
            if (request instanceof ServletServerHttpRequest) {
                setPerfTestAttributes((ServletServerHttpRequest) request, attributes);
            }
            return true;
        }

        @Override
        public void afterHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Exception exception) {
            //do nothing
        }

        private void setPerfTestAttributes(ServletServerHttpRequest request, Map<String, Object> attributes) {
            ServletServerHttpRequest req = request;
            if(PerfTestUtils.isPerfTestRequest(req.getServletRequest())){
                attributes.put(PerfTestUtils.PERF_TEST_KEY, true);
                String sceneId = PerfTestUtils.getPerfTestSceneId(req.getServletRequest());
                boolean isTestCluster = PerfTestUtils.isPerfTestCluster(req.getServletRequest());
                if (StringUtils.isNotBlank(sceneId)) {
                    attributes.put(PerfTestUtils.PERF_TEST_SCENE_ID_KEY, sceneId);
                    attributes.put(PerfTestUtils.PERF_TEST_CLUSTER, String.valueOf(isTestCluster));
                }
            }
        }
    }

    /**
     * websocket 请求加入压测标记(只支持spring-websocket的socketjs messagebroker开发方式)
     * @return
     */
    @Bean
    public AbstractWebSocketMessageBrokerConfigurer bootWebSocketMessageBrokerConfigurer(){
        return new AbstractWebSocketMessageBrokerConfigurer() {
            @Override
            public void registerStompEndpoints(StompEndpointRegistry registry) {
                //do nothing
            }

            @Override
            public void configureClientInboundChannel(ChannelRegistration registration) {
                registration.interceptors(new ExecutorChannelInterceptor(){

                    @Override
                    public Message<?> preSend(Message<?> message, MessageChannel channel) {
                        return message;
                    }
                    @Override
                    public void postSend(Message<?> message, MessageChannel channel, boolean sent) {
                        //do nothing
                    }
                    @Override
                    public void afterSendCompletion(Message<?> message, MessageChannel channel, boolean sent, Exception ex) {
                        //do nothing
                    }
                    @Override
                    public boolean preReceive(MessageChannel channel) {
                        return true;
                    }
                    @Override
                    public Message<?> postReceive(Message<?> message, MessageChannel channel) {
                        return message;
                    }
                    @Override
                    public void afterReceiveCompletion(Message<?> message, MessageChannel channel, Exception ex) {
                        //do nothing
                    }

                    // sockjs（websocket）事件处理前被调用
                    @Override
                    public Message<?> beforeHandle(Message<?> message, MessageChannel channel, MessageHandler handler) {
                        Map<String, Object> sessionAttr = (Map<String, Object>)message.getHeaders().get(SimpMessageHeaderAccessor.SESSION_ATTRIBUTES);
                        if(sessionAttr != null){
                            if(Boolean.TRUE.equals(sessionAttr.get(PerfTestUtils.PERF_TEST_KEY))){
                                String perfTestSceneId = (String) sessionAttr.get(PerfTestUtils.PERF_TEST_SCENE_ID_KEY);
                                boolean isTestCluster = Boolean.valueOf((String) sessionAttr.get(PerfTestUtils.PERF_TEST_CLUSTER));
                                InternalPerfTestContext.markAsPerfTest(perfTestSceneId, isTestCluster);
                            }
                        }
                        return message;
                    }
                    // sockjs（websocket）事件处理后被调用
                    @Override
                    public void afterMessageHandled(Message<?> message, MessageChannel channel, MessageHandler handler, Exception ex) {
                        InternalPerfTestContext.markAsNormal();
                    }
                });
            }

        };
    }

}

==> spring-boot-ext-1.3.60-sources.jar: cn/com/duibaboot/ext/autoconfigure/perftest/WebSocketPerfTestConfiguration.java <==
package cn.com.duibaboot.ext.autoconfigure.perftest;

import cn.com.duiba.boot.perftest.InternalPerfTestContext;
import cn.com.duiba.boot.perftest.PerfTestUtils;
import cn.com.duibaboot.ext.autoconfigure.core.SpecifiedBeanPostProcessor;
import com.alibaba.ttl.TransmittableThreadLocal;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeansException;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnResource;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;
import org.springframework.messaging.simp.SimpMessageHeaderAccessor;
import org.springframework.messaging.simp.config.ChannelRegistration;
import org.springframework.messaging.support.ExecutorChannelInterceptor;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.WebSocketMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.config.annotation.AbstractWebSocketMessageBrokerConfigurer;
import org.springframework.web.socket.config.annotation.StompEndpointRegistry;
import org.springframework.web.socket.config.annotation.WebSocketTransportRegistration;
import org.springframework.web.socket.handler.WebSocketHandlerDecorator;
import org.springframework.web.socket.handler.WebSocketHandlerDecoratorFactory;
import org.springframework.web.socket.server.HandshakeInterceptor;
import org.springframework.web.socket.server.support.WebSocketHandlerMapping;
import org.springframework.web.socket.server.support.WebSocketHttpRequestHandler;
import org.springframework.web.socket.sockjs.support.SockJsHttpRequestHandler;
import org.springframework.web.socket.sockjs.transport.TransportHandlingSockJsService;

import java.util.Map;

/**
 * websocket线上压力测试自动配置
 *
 * Created by wenqi.huang on 2016/11/24.
 */
@Configuration
@ConditionalOnResource(resources = "/autoconfig/perftest.properties")
@ConditionalOnClass({TransmittableThreadLocal.class,
        WebSocketHandlerMapping.class,
        AbstractWebSocketMessageBrokerConfigurer.class})
@Slf4j
public class WebSocketPerfTestConfiguration {

    /**
     * websocket 请求加入压测标记(只支持spring-websocket的socketjs messagebroker开发方式)
     * @return
     */
    @Bean
    public static SpecifiedBeanPostProcessor<WebSocketHandlerMapping> webSocketHandlerMappingSpecifiedBeanPostProcessor(){//NOSONAR
        return new SpecifiedBeanPostProcessor<WebSocketHandlerMapping>() {
            @Override
            public Class<WebSocketHandlerMapping> getBeanType() {
                return WebSocketHandlerMapping.class;
            }

            @Override
            public Object postProcessBeforeInitialization(WebSocketHandlerMapping bean, String beanName) throws BeansException {
                return bean;
            }

            @Override
            public Object postProcessAfterInitialization(WebSocketHandlerMapping bean, String beanName) throws BeansException {
                for(Object rh : bean.getUrlMap().values()){
                    if (rh instanceof SockJsHttpRequestHandler) {
                        SockJsHttpRequestHandler rh1 = (SockJsHttpRequestHandler)rh;
                        ((TransportHandlingSockJsService) rh1.getSockJsService())
                                .getHandshakeInterceptors().add(0, new PerfTestHandshakeInterceptor());
                    } else if(rh instanceof WebSocketHttpRequestHandler) {
                        WebSocketHttpRequestHandler rh1 = (WebSocketHttpRequestHandler)rh;
                        rh1.getHandshakeInterceptors().add(0, new PerfTestHandshakeInterceptor());
                    } else {
                        log.warn("暂时不支持对此websocket处理类：【{}】的压测, 请使用sockjs或stomp", rh.getClass().toString());
                    }
                }
                return bean;
            }

            @Override
            public int getOrder() {
                return 0;
            }
        };
    }

    public static class PerfTestHandshakeInterceptor implements HandshakeInterceptor {
        @Override
        public boolean beforeHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Map<String, Object> attributes) throws Exception {
            if (request instanceof ServletServerHttpRequest) {
                setPerfTestAttributes((ServletServerHttpRequest) request, attributes);
            }
            return true;
        }

        @Override
        public void afterHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Exception exception) {
            //do nothing
        }

        private void setPerfTestAttributes(ServletServerHttpRequest request, Map<String, Object> attributes) {
            ServletServerHttpRequest req = request;
            if(PerfTestUtils.isPerfTestRequest(req.getServletRequest())){
                attributes.put(PerfTestUtils.PERF_TEST_KEY, true);
                String sceneId = PerfTestUtils.getPerfTestSceneId(req.getServletRequest());
                boolean isTestCluster = PerfTestUtils.isPerfTestCluster(req.getServletRequest());
                if (StringUtils.isNotBlank(sceneId)) {
                    attributes.put(PerfTestUtils.PERF_TEST_SCENE_ID_KEY, sceneId);
                    attributes.put(PerfTestUtils.PERF_TEST_CLUSTER, String.valueOf(isTestCluster));
                }
            }
        }
    }

    /**
     * websocket 请求加入压测标记(只支持spring-websocket的socketjs messagebroker开发方式)
     * @return
     */
    @Bean
    public AbstractWebSocketMessageBrokerConfigurer bootWebSocketMessageBrokerConfigurer() { //NOSONAR
        return new AbstractWebSocketMessageBrokerConfigurer() {
            @Override
            public void registerStompEndpoints(StompEndpointRegistry registry) {
                //do nothing
            }

            @Override
            public void configureWebSocketTransport(WebSocketTransportRegistration registration) {
                registration.addDecoratorFactory(new WebSocketHandlerDecoratorFactory() {
                    @Override
                    public WebSocketHandler decorate(WebSocketHandler handler) {
                        return new WebSocketHandlerDecorator(handler){
                            @Override
                            public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {
                                try {
                                    Map<String, Object> sessionAttr = session.getAttributes();
                                    if(sessionAttr != null){
                                        if(Boolean.TRUE.equals(sessionAttr.get(PerfTestUtils.PERF_TEST_KEY))){
                                            String perfTestSceneId = (String) sessionAttr.get(PerfTestUtils.PERF_TEST_SCENE_ID_KEY);
                                            boolean isTestCluster = Boolean.valueOf((String) sessionAttr.get(PerfTestUtils.PERF_TEST_CLUSTER));
                                            InternalPerfTestContext.markAsPerfTest(perfTestSceneId, isTestCluster);
                                        }
                                    }

                                    super.handleMessage(session, message);
                                } finally {
                                    InternalPerfTestContext.markAsNormal();
                                }
                            }
                        };
                    }
                });
            }

            @Override
            public void configureClientInboundChannel(ChannelRegistration registration) {
                registration.interceptors(new ExecutorChannelInterceptor(){

                    @Override
                    public Message<?> preSend(Message<?> message, MessageChannel channel) {
                        return message;
                    }
                    @Override
                    public void postSend(Message<?> message, MessageChannel channel, boolean sent) {
                        //do nothing
                    }
                    @Override
                    public void afterSendCompletion(Message<?> message, MessageChannel channel, boolean sent, Exception ex) {
                        //do nothing
                    }
                    @Override
                    public boolean preReceive(MessageChannel channel) {
                        return true;
                    }
                    @Override
                    public Message<?> postReceive(Message<?> message, MessageChannel channel) {
                        return message;
                    }
                    @Override
                    public void afterReceiveCompletion(Message<?> message, MessageChannel channel, Exception ex) {
                        //do nothing
                    }

                    // sockjs（websocket）事件处理前被调用
                    @Override
                    public Message<?> beforeHandle(Message<?> message, MessageChannel channel, MessageHandler handler) {
                        Map<String, Object> sessionAttr = (Map<String, Object>)message.getHeaders().get(SimpMessageHeaderAccessor.SESSION_ATTRIBUTES);
                        if(sessionAttr != null){
                            if(Boolean.TRUE.equals(sessionAttr.get(PerfTestUtils.PERF_TEST_KEY))){
                                String perfTestSceneId = (String) sessionAttr.get(PerfTestUtils.PERF_TEST_SCENE_ID_KEY);
                                boolean isTestCluster = Boolean.valueOf((String) sessionAttr.get(PerfTestUtils.PERF_TEST_CLUSTER));
                                InternalPerfTestContext.markAsPerfTest(perfTestSceneId, isTestCluster);
                            }
                        }
                        return message;
                    }
                    // sockjs（websocket）事件处理后被调用
                    @Override
                    public void afterMessageHandled(Message<?> message, MessageChannel channel, MessageHandler handler, Exception ex) {
                        InternalPerfTestContext.markAsNormal();
                    }
                });
            }

        };
    }

}
